@@ -18,11 +18,21 @@ This module implements the initial value problem as a class.
 
     # Integrate this problem with a Runge-Kutta method
     >>> rk4 = rk.loadRKM('RK44')
-    >>> t,y = rk4(myivp)
-    >>> y[-1]
-    2.0611536225268342e-09
+    >>> t, y = rk4(myivp)
+    >>> y[-1][0] # doctest: +ELLIPSIS
+    2.06115362252...e-09
+
+    # Load a large set of non-stiff problems
+    >>> problems = detest_suite_plus()
+
+    # Load a few stiff problems
+    >>> problems = detest_stiff_suite()
 """
+from __future__ import print_function
+
+from __future__ import absolute_import
 import numpy as np
+from six.moves import range
 
 class IVP(object):
     """
@@ -37,75 +47,92 @@ class IVP(object):
             exact: a function that takes one argument (t) and returns
                    the exact solution (Should we make this a function of
                    u0 as well?)
-            dt0: The default initial timestep when a variable step size 
+            dt0: The default initial timestep when a variable step size
                  integrator is used.
             Any other problem-specific parameters.
 
     """
-    def __init__(self,f=None,u0=1.,T=1.):
+    def __init__(self, f=None, u0=1., t0=0., T=1., exact=None, desc='', name=''):
         self.u0  = u0
         self.rhs = f
         self.T   = T
+        self.exact = exact
+        self.description = desc
+        self.t0 = t0
+        self.dt0 = 1.e-2 # This doesn't really belong here
+        self.name = name
 
     def __repr__(self):
-        try:
-            return 'Problem Name:  '+self.name+'\n'+'Description:   '+self.description
-        except:
-            try:
-                return 'Problem Name: '+self.name
-            except:
-                return 'No name specified for this problem.'
+        return 'Problem Name:  '+self.name+'\n'+'Description:   '+self.description
 
-def load_ivp(ivpname):
-    """Load some very simple initial value problems."""
-    import numpy as np
+def load_ivp(ivpname='All'):
+    r"""Load some very simple initial value problems.  The following methods
+        are available::
+
+        >>> from nodepy import ivp
+        >>> ivps = ivp.load_ivp('all')
+        >>> for problem in sorted(ivps.values(), key=lambda x: x.name):
+        ...     print("{}: {}".format(problem.name, problem.description).rstrip())
+        2odes:
+        nlsin: A simple nonlinear scalar problem
+        ode1:
+        ode2:
+        test: Dahlquist's test problem; $f(y) = \lambda y$
+        vdp: Van der Pol oscillator with epsilon = 1/10.
+        zoltan: The linear scalar test problem with abs value
+    """
+    ivps = {}
+
+    name = 'test'
+    description = r"Dahlquist's test problem; $f(y) = \lambda y$"
+    u0 = 1.
+    rhs = lambda t,u: u
+    exact = lambda t, u0=u0: u0*np.exp(t)
+    ivps[name] = IVP(f=rhs, u0=u0, T=5., desc=description, exact=exact, name=name)
+
+    name = 'zoltan'
+    u0 = 1.
+    rhs = lambda t,u: -100*abs(u)
+    exact = lambda t, u0=u0: u0*np.exp(-100*t)
+    description = 'The linear scalar test problem with abs value'
+    ivps[name] = IVP(f=rhs, u0=u0, T=10., desc=description, exact=exact, name=name)
+
+    name = 'nlsin'
+    u0 = 1.
+    rhs = lambda t,u: 4.*u*float(np.sin(t))**3*np.cos(t)
+    exact = lambda t, u0=u0: u0*np.exp((np.sin(t))**4)
+    description = 'A simple nonlinear scalar problem'
+    ivps[name] = IVP(f=rhs, u0=u0, T=5., desc=description, exact=exact, name=name)
+
+    name = 'ode1'
+    rhs = lambda t,u: 4.*t*np.sqrt(u)
+    exact = lambda t: (1.+t**2)**2
+    ivps[name] = IVP(f=rhs, u0=1., T=5., exact=exact, name=name)
+
+    name = 'ode2'
+    u0=np.exp(1.)
+    rhs = lambda t,u: u/t*np.log(u)
+    exact = lambda t: np.exp(2.*t)
+    ivps[name] = IVP(f=rhs, u0=u0, t0=0.5, T=5., exact=exact, name=name)
+
+    name = '2odes'
+    u0=np.array([1.,1.])
+    rhs = lambda t,u: np.array([u[0], 2.*u[1]])
+    exact = lambda t: np.array([np.exp(t), np.exp(2.*t)])
+    ivps[name] = IVP(f=rhs, u0=u0, T=5., exact=exact, name=name)
+
+    name = 'vdp'
+    eps=0.1
+    u0=np.array([2.,-0.65])
+    rhs = lambda t,u: np.array([u[1], 1./eps*(-u[0]+(1.-u[0]**2)*u[1])])
+    description = 'Van der Pol oscillator with epsilon = 1/10.'
+    ivps[name] = IVP(f=rhs, u0=u0, T=5., exact=exact, name=name, desc=description)
+
+    if ivpname.lower() == 'all':
+        return ivps
+    else:
+        return ivps[ivpname]
 
-    ivp=IVP()
-    if ivpname=='test':
-        ivp.u0=1.
-        ivp.rhs = lambda t,u: u
-        ivp.exact = lambda t : ivp.u0*np.exp(t)
-        ivp.T = 5.
-        ivp.description = 'The linear scalar test problem'
-    elif ivpname=='zoltan':
-        ivp.u0=1.
-        ivp.rhs = lambda t,u: -100*abs(u)
-        ivp.exact = lambda t : ivp.u0*np.exp(-100*t)
-        ivp.T = 10.
-        ivp.description = 'The linear scalar test problem with abs value'
-    elif ivpname=='nlsin':
-        ivp.u0=1.
-        ivp.rhs = lambda t,u: 4.*u*float(np.sin(t))**3*np.cos(t)
-        ivp.exact = lambda t: ivp.u0*np.exp((np.sin(t))**4)
-        ivp.T = 5.
-        ivp.dt0=1.e-2
-        ivp.description = 'A simple nonlinear scalar problem'
-    elif ivpname=='ode1':
-        ivp.u0=1.
-        ivp.rhs = lambda t,u: 4.*t*np.sqrt(u)
-        ivp.exact = lambda t: (1.+t**2)**2
-        ivp.T = 5.
-    elif ivpname=='ode2':
-        ivp.u0=np.exp(1.)
-        ivp.t0=0.5
-        ivp.rhs = lambda t,u: u/t*np.log(u)
-        ivp.exact = lambda t: np.exp(2.*t)
-        ivp.T = 5.
-    elif ivpname=='2odes':
-        ivp.u0=np.array([1.,1.])
-        ivp.rhs = lambda t,u: np.array([u[0], 2.*u[1]])
-        ivp.exact = lambda t: np.array([np.exp(t), np.exp(2.*t)])
-        ivp.T = 5.
-    elif ivpname=='vdp':
-        ivp.eps=0.1
-        ivp.u0=np.array([2.,-0.65])
-        ivp.rhs = lambda t,u: np.array([u[1], 1./ivp.eps*(-u[0]+(1.-u[0]**2)*u[1])])
-        ivp.T = 5.
-        ivp.dt0=1.e-2
-        ivp.description = 'The van der Pol oscillator'
-    else: print 'Unknown IVP name; returning empty IVP'
-    ivp.name=ivpname
-    return ivp
 
 def detest(testkey):
     """
@@ -125,10 +152,10 @@ def detest(testkey):
             standard, and although there are certain dangers in accepting
             any particular set of problems as a universal standard, it
             is nevertheless sometimes useful to try a new method on this
-            test set due to the availability of published results for 
+            test set due to the availability of published results for
             many existing methods.
-        
-        Reference: [enright1987]_ 
+
+        Reference: :cite:`enright1987`.
     """
     import numpy as np
     ivp=IVP()
@@ -195,7 +222,7 @@ def detest(testkey):
         e=np.arange(1,11); e[-1]=0.
         ivp.L_rhs = np.diag(-e)+np.diag(e[:-1],-1);
         ivp.rhs = lambda t,u: np.dot(ivp.L_rhs,u)
-        ivp.dt0 = 1.e-2    
+        ivp.dt0 = 1.e-2
     elif testkey=='C3':
         ivp.u0=np.zeros(10); ivp.u0[0]=1.
         ivp.T=20.
@@ -209,10 +236,17 @@ def detest(testkey):
         e=np.ones(51)
         ivp.L_rhs = np.diag(-2*e)+np.diag(e[:-1],-1)+np.diag(e[:-1],1);
         ivp.rhs = lambda t,u: np.dot(ivp.L_rhs,u)
-        ivp.dt0 = 1.e-2     
+        ivp.dt0 = 1.e-2
     elif testkey=='C5':
         ivp.u0=np.zeros(30)
-	ivp.u0 = np.array([3.42947415189,3.35386959711,1.35494901715,6.64145542550,5.97156957878,2.18231499728,11.2630437207,14.6952576794,6.27960525067,-30.1552268759,1.65699966404,1.43785752721,-21.1238353380,28.4465098142,15.3882659679,-.557160570446,.505696783289,.230578543901,-.415570776342,.365682722812,.169143213293,-.325325669158,.189706021964,.0877265322780,-.0240476254170,-.287659532608,-.117219543175,-.176860753121,-.216393453025,-.0148647893090])	
+        ivp.u0 = np.array([3.42947415189, 3.35386959711, 1.35494901715,
+            6.64145542550, 5.97156957878, 2.18231499728, 11.2630437207,
+            14.6952576794, 6.27960525067, -30.1552268759, 1.65699966404,
+            1.43785752721, -21.1238353380, 28.4465098142, 15.3882659679,
+            -0.557160570446, 0.505696783289, 0.230578543901, -0.415570776342,
+            0.365682722812, 0.169143213293, -0.325325669158, 0.189706021964,
+            0.0877265322780, -0.0240476254170, -0.287659532608, -0.117219543175,
+            -0.176860753121,  -0.216393453025, -0.0148647893090])
         ivp.T=20.
         ivp.rhs = _C5rhs
         ivp.dt0 = 1.e-2
@@ -352,7 +386,7 @@ def _F4rhs(t,u):
 def _F5rhs(t,u):
     du=np.zeros(1)
     c=np.sum([i**(4./3) for i in range(1,20)])
-    du[0] = 4./3/c * np.sum([(t-i+0j)**(4./3) for i in range(1,20)])*u[0]
+    du[0] = 4./3/c * np.sum([(t-i+0j)**(4./3) for i in range(1,20)]).real*u[0]
     return du
 
 def _B4rhs(t,u):
@@ -363,37 +397,37 @@ def _B4rhs(t,u):
     return du
 def _C5rhs(t,u):
     r = np.zeros(5); u1 = np.zeros(15); u2 = np.zeros(15); y1 = np.zeros((3,5)); y2 = np.zeros((3,5)); d = np.zeros((5,5)); du=np.zeros(30); ynew1 = np.zeros((3,5)); ynew2 = np.zeros((3,5));
-    k2 = 2.95912208286; m0 = 1.00000597682; 
+    k2 = 2.95912208286; m0 = 1.00000597682;
     m = np.array([.000954786104043,.000285583733151,.0000437273164546,.0000517759138449,.00000277777777778])
     k=0
     for j in range(0,5):
-	for i in range(0,3):
-	    y1[i][j] = u[k]
-	    y2[i][j] = u[k+15]
-	    k+=1
+        for i in range(0,3):
+            y1[i][j] = u[k]
+            y2[i][j] = u[k+15]
+            k+=1
 
     for j in range(0,5):
-	r[j] = np.sqrt(y1[0][j]**2+y1[1][j]**2+y1[2][j]**2)
+        r[j] = np.sqrt(y1[0][j]**2+y1[1][j]**2+y1[2][j]**2)
 
     for k in range(0,5):
-	for j in range(0,5):
-	    d[k][j] = np.sqrt((y1[0][k]-y1[0][j])**2+(y1[1][k]-y1[1][j])**2+(y1[2][k]-y1[2][j])**2)
-	      
+        for j in range(0,5):
+            d[k][j] = np.sqrt((y1[0][k]-y1[0][j])**2+(y1[1][k]-y1[1][j])**2+(y1[2][k]-y1[2][j])**2)
+
     for j  in range(0,5):
-	for i in range(0,3):
-	    term=0.;
-	    for k in range(0,5):
-		if (k != j):
-		    term = term+m[k]*((y1[i][k]-y1[i][j])/d[j][k]**3 - y1[i][k]/r[k]**3)
-	    ynew1[i][j] = y2[i][j] 
-	    ynew2[i][j] = k2*(-(m0+m[j])*y1[i][j]/r[j]**3+term)
+        for i in range(0,3):
+            term=0.;
+            for k in range(0,5):
+                if (k != j):
+                    term = term+m[k]*((y1[i][k]-y1[i][j])/d[j][k]**3 - y1[i][k]/r[k]**3)
+            ynew1[i][j] = y2[i][j]
+            ynew2[i][j] = k2*(-(m0+m[j])*y1[i][j]/r[j]**3+term)
     k=0
     for j in range(0,5):
-	for i in range(0,3):
-	    du[k] = ynew1[i][j]
-	    du[k+15] = ynew2[i][j] 
-	    k+=1
-    
+        for i in range(0,3):
+            du[k] = ynew1[i][j]
+            du[k+15] = ynew2[i][j]
+            k+=1
+
     return du
 
 def detest_suite():
@@ -411,6 +445,11 @@ def detest_suite_minus():
     detestkeys=['A1','A2','A3','A4','A5','B1','B2','B3','B4','B5','C1','C2','C3','C4','C5','D1','D2','D3','D4','D5','E1','E2','E3','E4','E5']
     return [detest(dtkey) for dtkey in detestkeys]
 
+def detest_stiff_suite():
+    """The entire non-stiff DETEST suite of problems."""
+    detestkeys=['A1', 'A2', 'A3', 'A4']
+    return [detest_stiff(dtkey) for dtkey in detestkeys]
+
 
 def detest_stiff(testkey):
     """The stiff DETEST suite.  Only a few problems have been implemented."""
@@ -435,7 +474,8 @@ def detest_stiff(testkey):
         ivp.T=1.
         ivp.rhs = _A4rhs_stiff
         ivp.dt0 = 1.e-5
-    else: print 'Unknown Detest problem; returning empty IVP'
+    else:
+        print('Unknown Detest problem; returning empty IVP')
     return ivp
 
 def _A2rhs_stiff(t,u):
