@@ -1,7 +1,11 @@
+from __future__ import print_function
+
+from __future__ import absolute_import
 import numpy as np
 from sympy import factorial, sympify, Rational
 #from sage.combinat.combinat import permutations
-from utils import permutations
+from nodepy.utils import permutations
+from six.moves import range
 
 #=====================================================
 class RootedTree(str):
@@ -9,14 +13,14 @@ class RootedTree(str):
     r"""
         A rooted tree is a directed acyclic graph with one node, which
         has no incoming edges, designated as the root.
-        Rooted trees are useful for analyzing the order conditions of 
-        multistage numerical ODE solvers, such as Runge-Kutta methods 
+        Rooted trees are useful for analyzing the order conditions of
+        multistage numerical ODE solvers, such as Runge-Kutta methods
         and other general linear methods.
 
         The trees are represented as strings, using one of the notations
         introduced by Butcher (the third column of Table 300(I) of
-        Butcher's text).  The character 'T' is used in place of $\\tau$ 
-        to represent a vertex, and braces '{ }' are used instead of brackets 
+        Butcher's text).  The character 'T' is used in place of $\\tau$
+        to represent a vertex, and braces '{ }' are used instead of brackets
         '[ ]' to indicate that everything inside the braces
         is joined to a single parent node.  Thus the first four trees are:
 
@@ -26,24 +30,24 @@ class RootedTree(str):
         returns a list of all trees of a given order::
 
             >>> from nodepy import *
-            >>> for p in range(4): print rt.list_trees(p)
+            >>> for p in range(4): print(rt.list_trees(p))
             ['']
             ['T']
             ['{T}']
             ['{{T}}', '{T^2}']
-            
+
         Note that the tree of order 0 is indicated by an empty string.
-        
+
         If the tree contains an edge from vertex A to vertex B, vertex
         B is said to be a child of vertex A.
-        A vertex with no children is referred to as a leaf.  
+        A vertex with no children is referred to as a leaf.
 
         .. warning::
 
-            One important convention is assumed in the code; namely, that at each 
-            level, leaves are listed first (before any other subtrees), 
+            One important convention is assumed in the code; namely, that at each
+            level, leaves are listed first (before any other subtrees),
             and if there are $n$ leaves, we write 'T^n'.
-        
+
         .. note::
 
             Currently, powers cannot be used for subtrees; thus
@@ -52,7 +56,7 @@ class RootedTree(str):
 
             is valid, while
 
-            '{{T}^2}' 
+            '{{T}^2}'
 
             is not.  This restriction may be lifted in the future.
 
@@ -79,9 +83,9 @@ class RootedTree(str):
             >>> rk.elementary_weight_str(tree)
             'dot(b,dot(A,c)*dot(A,c*dot(A,c))*c**2)'
 
-        **References**:  
-            #. [butcher2003]_
-            #. [hairer1993]_
+        **References**:
+            * :cite:`butcher2003`
+            * :cite:`hairer1993`
     """
     def __init__(self,strg):
         """
@@ -98,7 +102,7 @@ class RootedTree(str):
 
     def order(self):
         """
-        The order of a rooted tree, denoted $r(t)$, is the number of 
+        The order of a rooted tree, denoted $r(t)$, is the number of
         vertices in the tree.
 
         **Examples**::
@@ -108,7 +112,7 @@ class RootedTree(str):
             >>> tree.order()
             7
         """
-        from strmanip import getint
+        from nodepy.strmanip import getint
         if self=='T': return 1
         if self=='':  return 0
         r=self.count('{')
@@ -132,9 +136,7 @@ class RootedTree(str):
             >>> tree.density()
             56
 
-        **Reference**: 
-
-            - [butcher2003]_ p. 127, eq. 301(c)
+        **Reference**: :cite:`butcher2003` p. 127, eq. 301(c)
         """
         gamma=self.order()
         nleaves,subtrees=self._parse_subtrees()
@@ -143,7 +145,7 @@ class RootedTree(str):
         return gamma
 
     def symmetry(self):
-        r""" 
+        r"""
         The symmetry $\\sigma(t)$ of a rooted tree is...
 
         **Examples**::
@@ -153,11 +155,9 @@ class RootedTree(str):
             >>> tree.symmetry()
             2
 
-        **Reference**: 
-
-            - [butcher2003]_ p. 127, eq. 301(b)
+        **Reference**: :cite:`butcher2003` p. 127, eq. 301(b)
         """
-        from strmanip import getint
+        from nodepy.strmanip import getint
         if self=='T': return 1
         sigma=1
         if self[1]=='T':
@@ -173,17 +173,16 @@ class RootedTree(str):
 
 
     def Dmap(self):
-        """ 
+        """
         Butcher's function $D(t)$.  Represents differentiation.
         Defined by $D(t)$=0 except for D('T')=1.
 
-        **Reference**: 
-            #. [butcher1997]_
+        **Reference**: :cite:`butcher1997`
         """
         return self=='T'
 
     def lamda(self,alpha,extraargs=[]):
-        r""" 
+        r"""
             Computes Butcher's functional lambda on a given tree
             for the function alpha.  This is used to compute the
             product of two functions on trees.
@@ -191,15 +190,15 @@ class RootedTree(str):
             *INPUT*:
 
                 * alpha -- a function on rooted trees
-                * extraargs -- a list containing any additional arguments 
+                * extraargs -- a list containing any additional arguments
                   that must be passed to alpha
 
             *OUTPUT*:
                 * tprod -- a list of trees [t1, t2, ...]
                 * fprod -- a list of numbers [a1, a2, ...]
 
-            The meaning of the output is that 
-            $\\lambda(\\alpha,t)(\\beta)=a1*\\beta(t1)+a2*\\beta(t2)+...$ 
+            The meaning of the output is that
+            $\\lambda(\\alpha,t)(\\beta)=a1*\\beta(t1)+a2*\\beta(t2)+...$
 
             **Examples**::
 
@@ -208,8 +207,7 @@ class RootedTree(str):
                 >>> tree.lamda(rt.Emap)
                 (['T', '{T}', '{{T}}', '{T}', '{T^2}', '{T{T}}'], [1/2, 1, 1, 1/2, 1, 1])
 
-            **Reference**: 
-                [butcher2003]_ pp. 275-276
+            **Reference**: :cite:`butcher2003` pp. 275-276
         """
         if self=='': return [RootedTree('')],[0]
         if self=='T': return [RootedTree('T')],[1]
@@ -263,12 +261,12 @@ class RootedTree(str):
         return tprod,fprod
 
     def _factor(self):
-        """ 
+        """
             Returns two rooted trees, t and u, such that self=t*u.
 
-            **Input**: 
+            **Input**:
                 - self -- any rooted tree
-            **Output**: 
+            **Output**:
                 - t,u -- a pair of rooted trees whose product t*u is equal to self.
 
             **Examples**::
@@ -304,15 +302,15 @@ class RootedTree(str):
         return t,u
 
     def Gprod(self,alpha,beta,alphaargs=[],betaargs=[]):
-        r""" 
+        r"""
             Returns the product of two functions on a given tree.
 
             INPUT:
                 alpha, beta -- two functions on rooted trees
                                that return symbolic or numeric values
-                alphaargs   -- a string containing any additional arguments 
+                alphaargs   -- a string containing any additional arguments
                                 that must be passed to function alpha
-                betaargs    -- a string containing any additional arguments 
+                betaargs    -- a string containing any additional arguments
                                 that must be passed to function beta
 
             OUTPUT:
@@ -339,7 +337,7 @@ class RootedTree(str):
                 >>> tree.Gprod(rt.Emap,Dmap)
                 1/2
 
-            **Reference**: [butcher2003]_ p. 276, Thm. 386A 
+            **Reference**: :cite:`butcher2003` p. 276, Thm. 386A
         """
         trees,factors=self.lamda(alpha,*alphaargs)
         s=0
@@ -349,7 +347,7 @@ class RootedTree(str):
         return s
 
     def Gprod_str(self,alpha,beta,alphaargs=[],betaargs=[]):
-        """ 
+        """
             Alternate version of Gprod, but operates on strings.
             Hopefully can be eliminated later in favor of symbolic
             manipulation.
@@ -368,6 +366,29 @@ class RootedTree(str):
             s+="+"+str(sympify(alph+'*'+bet))
         return s
 
+    def _plot_subtree(self,xroot,yroot,xwidth):
+        """
+            Recursively plots subtrees.  Should only be called from plot().
+
+            INPUT:
+                xroot, yroot -- coordinates at which root of this subtree
+                                is plotted
+                xwidth -- width in which this subtree must fit, in order
+                            to avoid possibly overlapping with others
+        """
+        import matplotlib.pyplot as plt
+        ychild=yroot+1
+        nleaves,subtrees=self._parse_subtrees()
+        nchildren=nleaves+len(subtrees)
+
+        dist=xwidth*(nchildren-1)/2.
+        xchild=np.linspace(xroot-dist,xroot+dist,nchildren)
+        plt.scatter(xchild,ychild*np.ones(nchildren))
+        for i in range(nchildren):
+            plt.plot([xroot,xchild[i]],[yroot,ychild],'-k')
+            if i>nleaves-1:
+                subtrees[i-nleaves]._plot_subtree(xchild[i],ychild,xwidth/3.)
+
     def plot(self,nrows=1,ncols=1,iplot=1,ttitle=''):
         """
             Plots the rooted tree.
@@ -384,52 +405,25 @@ class RootedTree(str):
             *OUTPUT*: None.
 
             The plot is created recursively by
-            plotting the root, parsing the subtrees, plotting the 
+            plotting the root, parsing the subtrees, plotting the
             subtrees' roots, and calling _plot_subtree on each child
         """
-        import matplotlib.pyplot as pl
-        if iplot==1: pl.clf()
-        pl.subplot(nrows,ncols,iplot)
-        pl.hold(True)
-        pl.scatter([0],[0])
+        import matplotlib.pyplot as plt
+        if iplot==1: plt.clf()
+        plt.subplot(nrows,ncols,iplot)
+        plt.scatter([0],[0])
         if self!='T': self._plot_subtree(0,0,1.)
 
-        fs=int(np.ceil(30./nrows))
-        pl.title(ttitle,{'fontsize': fs})
-        pl.xticks([])
-        pl.yticks([])
-        pl.hold(False)
-        pl.axis('off')
-        #pl.show()
-        #pl.ioff()
+        fs=int(np.ceil(20./nrows))
+        plt.title(ttitle,{'fontsize': fs})
+        plt.xticks([])
+        plt.yticks([])
+        plt.axis('off')
 
 
-    def _plot_subtree(self,xroot,yroot,xwidth):
-        """
-            Recursively plots subtrees.  Should only be called from plot().
-
-            INPUT:
-                xroot, yroot -- coordinates at which root of this subtree 
-                                is plotted
-                xwidth -- width in which this subtree must fit, in order
-                            to avoid possibly overlapping with others
-        """
-        import matplotlib.pyplot as pl
-        ychild=yroot+1
-        nleaves,subtrees=self._parse_subtrees()
-        nchildren=nleaves+len(subtrees)
-
-        dist=xwidth*(nchildren-1)/2.
-        xchild=np.linspace(xroot-dist,xroot+dist,nchildren)
-        pl.scatter(xchild,ychild*np.ones(nchildren))
-        for i in range(nchildren):
-            pl.plot([xroot,xchild[i]],[yroot,ychild],'-k')
-            if i>nleaves-1:
-                subtrees[i-nleaves]._plot_subtree(xchild[i],ychild,xwidth/3.)
-
 
     def _parse_subtrees(self):
-        """ 
+        """
             Returns the number of leaves and a list of the subtrees,
             for a given rooted tree.
 
@@ -442,12 +436,12 @@ class RootedTree(str):
             returning possibly many copies of 'T', the leaves are just
             returned as a number.
         """
-        from strmanip import get_substring, open_to_close, getint
+        from nodepy.strmanip import get_substring, open_to_close, getint
         if str(self)=='T' or str(self)=='': return 0,[]
         pos=0
         #Count leaves at current level
-        if self[1]=='T':    
-            if self[2]=='^': 
+        if self[1]=='T':
+            if self[2]=='^':
                 nleaves=getint(self[3:])
             else: nleaves=1
         else: nleaves=0
@@ -462,13 +456,13 @@ class RootedTree(str):
         return nleaves,subtrees
 
     def list_equivalent_trees(self):
-        """ 
+        """
             Returns a list of all strings (subject to our assumptions)
-            equivalent to a given tree 
+            equivalent to a given tree
 
-            INPUT: 
+            INPUT:
                 self     -- any rooted tree
-            OUTPUT:  
+            OUTPUT:
                 treelist -- a list of all the 'legal' tree strings
                             that produce the same tree.
 
@@ -494,11 +488,11 @@ class RootedTree(str):
         else: return False
 
     def __mul__(self,tree2):
-        """ 
-            Returns Butcher's product: t*u is the tree obtained by
-            attaching the root of u as a child to the root of t. 
         """
-        from strmanip import getint
+            Returns Butcher's product: t*u is the tree obtained by
+            attaching the root of u as a child to the root of t.
+        """
+        from nodepy.strmanip import getint
         if self=='T': return RootedTree('{'+tree2+'}')
         if tree2=='T':  # We're just adding a leaf to self
             nleaves,subtrees=self._parse_subtrees()
@@ -515,8 +509,17 @@ class RootedTree(str):
 #=====================================================
 def plot_all_trees(p,title='str'):
 #=====================================================
-    """ Plots all rooted trees of order p """
-    import matplotlib.pyplot as pl
+    """ Plots all rooted trees of order p.
+
+        **Example**:
+
+        Plot all trees of order 4::
+
+            >>> from nodepy import rt
+            >>> rt.plot_all_trees(4)  # doctest: +ELLIPSIS
+            <Figure...
+    """
+    import matplotlib.pyplot as plt
     forest=list_trees(p)
     nplots=len(forest)
     nrows=int(np.ceil(np.sqrt(float(nplots))))
@@ -524,35 +527,35 @@ def plot_all_trees(p,title='str'):
     if nrows*ncols<nplots: ncols=ncols+1
     for tree in forest:
         if title=='str': ttitle=tree
-        else: ttitle=''
+        else: ttitle=title(tree)
         tree.plot(nrows,ncols,forest.index(tree)+1,ttitle=ttitle)
-    fig=pl.figure(1)
-    pl.setp(fig,facecolor='white')
+    fig=plt.figure(1)
+    plt.setp(fig,facecolor='white')
     return fig
 
 #=====================================================
 def list_trees(p,ind='all'):
 #=====================================================
-    """ 
+    """
     Returns rooted trees of order p.
 
-    INPUT: 
-    
+    INPUT:
+
         - p   -- order of trees desired
         - ind -- if given, returns a single tree corresponding to this index.
                 Not very useful since the ordering isn't obvious.
-                
+
     OUTPUT: list of all trees of order p (or just one, if ind is provided).
-    
+
     Generates the rooted trees using Albrecht's 'Recursion 3'.
 
     **Examples**:
 
     Produce column of Butcher's Table 302(I)::
 
-        >>> for i in range(1,11): 
+        >>> for i in range(1,11):
         ...     forest=list_trees(i)
-        ...     print len(forest)
+        ...     print(len(forest))
         1
         1
         2
@@ -566,15 +569,17 @@ def list_trees(p,ind='all'):
 
     .. warning::
 
-        This code is complete only up to order 10.  We need to extend it 
+        This code is complete only up to order 10.  We need to extend it
         by adding more subloops for p>10.
 
     TODO: Implement Butcher's formula (Theorem 302B) for the number
             of trees and determine to what order this is valid.
 
-    **Reference**: [albrecht1996]_
+    **Reference**: :cite:`albrecht1996`
     """
 
+    if p>10: raise Exception('list_trees is not complete for orders p > 10.')
+
     if p==0: return [RootedTree('')]
     W=[[],[]] #This way indices agree with Albrecht
     R=[[],[]]
@@ -637,10 +642,10 @@ def list_trees(p,ind='all'):
     for i in range(1,p):
         ps=_powerString("T",i,powchar="^")
         W[i].append(RootedTree("{"+ps+"}"))
-  
+
     if ind=='all': return W[p-1]
     else: return W[p-1][ind]
-    
+
 
 def _powerString(s,npow,powchar="**",trailchar=''):
     r"""Raise string s to power npow with additional formatting."""
@@ -689,7 +694,7 @@ def Dprod_str(tree,alpha):
     return result
 
 def Dmap(tree):
-    """ 
+    """
     Butcher's function D(t).  Represents differentiation.
     Defined by D(t)=0 except for D('T')=1.
     """
@@ -707,7 +712,7 @@ def Gprod_str(tree,alpha,beta,alphaargs='',betaargs=[]):
     return tree.Gprod_str(alpha,beta,alphaargs,betaargs)
 
 def Emap(tree,a=1):
-    """ 
+    """
     Butcher's function E^a(t).
     Gives the B-series for the exact solution advanced 'a' steps
     in time.
@@ -721,9 +726,7 @@ def Emap(tree,a=1):
         >>> rt.Emap(tree,a=2)
         16/7
 
-    **Reference**: 
-
-        [butcher1997]_
+    **Reference**: :cite:`butcher1997`
     """
     return Rational(a**tree.order(),(tree.density()))
 
@@ -734,7 +737,7 @@ def Emap_str(tree,a=1):
 #=====================================================
 def recursiveVectors(p,ind='all'):
 #=====================================================
-  """ 
+  """
     Generate recursive vectors using Albrecht's 'recursion 1'.
     These are essentially the order conditions for Runge-Kutta
     methods, excluding those that correspond to bushy trees.
@@ -746,14 +749,23 @@ def recursiveVectors(p,ind='all'):
     approach.  But as a set of conditions up to some order they
     are, of course, equivalent.
 
-    Follows [albrecht1996]_ p. 1718
+    Follows :cite:`albrecht1996` p. 1718
 
     .. warning::
 
-        This code is complete only up to order 12.  We need to extend it 
-        by adding more subloops for p>12.
+        This code is complete only up to order 14.  We need to extend it
+        by adding more subloops for p>14.
+
+    **Example**
+
+    Count number of conditions for order 12::
+
+        >>> from nodepy import rt
+        >>> v = rt.recursiveVectors(12)
+        >>> print(len(v))
+        4765
   """
-  if p>12: print 'recursiveVectors is not complete for orders p>12.'
+  if p>14: raise Exception('recursiveVectors is not complete for orders p > 14.')
   W=[[],[]]
   R=[[],[]]
   R.append(["tau[2]"])
@@ -761,21 +773,23 @@ def recursiveVectors(p,ind='all'):
   for i in range(3,p):
      #Construct R[i]
      R.append(["tau["+str(i)+"]"])
-     for j in range(len(W[i-1])):
-        R[i].append("A,"+W[i-1][j])
+     for w in W[i-1]:
+        R[i].append("A,"+w)
      #Construct W[i]
      #l=0:
      W.append(R[i][:])
      for l in range(1,i-1): #level 1
         ps=_powerString("C",l,trailchar=",")
-        for j in range(len(R[i-l])):
-          W[i].append(ps+R[i-l][j])
+        for r in R[i-l]:
+          W[i].append(ps+r)
      for l in range(0,i-3): #level 2
         ps=_powerString("C",l,trailchar=",")
         for n in range(2,i-l-1):
           m=i-n-l
           if m<=n: #Avoid duplicate conditions
              for Rm in R[m]:
+                # if m<n, start from R[0]
+                # if m==n, start from Rm
                 lowlim=(m<n and [0] or [R[m].index(Rm)])[0]
                 for Rn in R[n][lowlim:]:
                   W[i].append(ps+Rm+"*"+Rn)
@@ -804,7 +818,7 @@ def recursiveVectors(p,ind='all'):
                         lowlim2=(n<s and [0] or [R[n].index(Rn)])[0]
                         for Rs in R[s][lowlim2:]:
                           lowlim3=(s<t and [0] or [R[s].index(Rs)])[0]
-                          for Rt in R[t]:
+                          for Rt in R[t][lowlim3:]:
                              W[i].append(ps+Rm+"*"+Rn+"*"+Rs+"*"+Rt)
      for l in range(0,i-9): # level 5
         ps=_powerString("C",l,trailchar=",")
@@ -820,11 +834,64 @@ def recursiveVectors(p,ind='all'):
                             lowlim2=(n<s and [0] or [R[n].index(Rn)])[0]
                             for Rs in R[s][lowlim2:]:
                               lowlim3=(s<t and [0] or [R[s].index(Rs)])[0]
-                              for Rt in R[t]:
+                              for Rt in R[t][lowlim3:]:
                                   lowlim4=(t<u and [0] or [R[t].index(Rt)])[0]
-                                  for Ru in R[u]:
+                                  for Ru in R[u][lowlim4:]:
                                      W[i].append(ps+Rm+"*"+Rn+"*"+Rs+"*"+Rt+"*"+Ru)
 
+     for l in range(0,i-11): # level 6
+        ps=_powerString("C",l,trailchar=",")
+        for m in range(2,i-l-9):
+          for n in range(2,i-l-m-7):
+             for s in range(2,i-l-n-m-5):
+               for t in range(2,i-l-n-m-s-3):
+                 for u in range(2,i-l-n-m-s-t-1):
+                    v=i-t-s-m-n-l-u
+                    if m<=n<=s<=t<=u<=v: #Avoid duplicate conditions
+                      for Rm in R[m]:
+                        lowlim=(m<n and [0] or [R[m].index(Rm)])[0]
+                        for Rn in R[n][lowlim:]:
+                          lowlim2=(n<s and [0] or [R[n].index(Rn)])[0]
+                          for Rs in R[s][lowlim2:]:
+                            lowlim3=(s<t and [0] or [R[s].index(Rs)])[0]
+                            for Rt in R[t][lowlim3:]:
+                              lowlim4=(t<u and [0] or [R[t].index(Rt)])[0]
+                              for Ru in R[u][lowlim4:]:
+                                  lowlim5=(u<v and [0] or [R[u].index(Ru)])[0]
+                                  for Rv in R[v][lowlim5:]:
+                                    W[i].append(ps+Rm+"*"+Rn+"*"+Rs+"*"+Rt+"*"+Ru+"*"+Rv)
+
+# It seems like the code above should give correct values for order up to 16,
+# but it does not agree with OEIS sequence A000081 starting at order 15.
+# I'm not sure what's wrong; it may be that this approach leads to duplicate conditions
+# starting only at order 15 (see comment on p. 1719 of Albrecht's paper).
+
+# The code below seems correct but in light of the issue above there is no point in using it.
+#     for l in range(0,i-13): # level 7
+#        ps=_powerString("C",l,trailchar=",")
+#        for n in range(2,i-l-11):
+#          for m in range(2,i-l-n-9):
+#             for s in range(2,i-l-n-m-7):
+#               for t in range(2,i-l-n-m-s-5):
+#                 for u in range(2,i-l-n-m-s-t-3):
+#                   for v in range(2,i-l-n-m-s-t-u-1):
+#                     x=i-t-s-m-n-l-u
+#                     if m<=n<=s<=t<=u<=v<=x: #Avoid duplicate conditions
+#                       for Rm in R[m]:
+#                         lowlim=(m<n and [0] or [R[m].index(Rm)])[0]
+#                         for Rn in R[n][lowlim:]:
+#                           lowlim2=(n<s and [0] or [R[n].index(Rn)])[0]
+#                           for Rs in R[s][lowlim2:]:
+#                             lowlim3=(s<t and [0] or [R[s].index(Rs)])[0]
+#                             for Rt in R[t][lowlim3:]:
+#                               lowlim4=(t<u and [0] or [R[t].index(Rt)])[0]
+#                               for Ru in R[u][lowlim4:]:
+#                                   lowlim5=(u<v and [0] or [R[u].index(Ru)])[0]
+#                                   for Rv in R[v][lowlim5:]:
+#                                     lowlim6=(v<x and [0] or [R[v].index(Rv)])[0]
+#                                     for Rx in R[x][lowlim5:]:
+#                                       W[i].append(ps+Rm+"*"+Rn+"*"+Rs+"*"+Rt+"*"+Ru+"*"+Rv+"*"+Rx)
+
   if ind=='all': return W[p-1]
   else: return W[p-1][ind]
 
